"""
Main invocation for spelling check
"""
from __future__ import absolute_import, division, print_function

import json
import os
import pathlib

import pyspelling

from spelling.config import get_config_context_manager


def check(  # pylint: disable=too-many-arguments
    display_context, display_summary, display_help, config, fobj, jsonfobj=None
):
    """
    Execute the invocation
    """
    workingpath = pathlib.Path(".").resolve()
    success = True
    check_iter_output = check_iter(
        display_context,
        display_summary,
        display_help,
        config,
        workingpath,
        jsonfobj=jsonfobj,
    )
    for output in check_iter_output:
        print(output, file=fobj)
        success = False
    if success:
        print("Spelling check passed :)", file=fobj)
    return success


def check_iter(  # pylint: disable=too-many-arguments
    display_context, display_summary, display_help, config, workingpath, jsonfobj=None
):
    """
    Execute the invocation
    """
    all_results, custom_wordlists = run_spell_check(config, workingpath)
    if jsonfobj is not None:
        all_results = list(all_results)
        json.dump(results_to_json(all_results), jsonfobj)
    yield from process_results(
        all_results, custom_wordlists, display_context, display_summary, display_help
    )


def results_to_json(all_results):
    """
    Save the result format in a machine processable format.
    """
    words = {}
    for results in all_results:
        if results.words:
            for word in results.words:
                filename = context_to_filename(results.context)
                words.setdefault(word, {}).setdefault("files", []).append(
                    {"category": results.category, "file": filename}
                )
    return words


def context_to_filename(name):
    """
    Turn a context line into the filepath.
    """
    testname = name
    if os.path.isfile(testname):
        return testname
    testname = testname.split(":", 1)[0]
    if os.path.isfile(testname):
        return testname
    testname = testname.rsplit("(", 1)[0]
    if os.path.isfile(testname):
        return testname
    testname = testname.strip()
    if os.path.isfile(testname):
        return testname
    raise Exception(f"Unable to get filepath for {name}")


def process_results(
    all_results, custom_wordlists, display_context, display_summary, display_help
):
    """
    Work through the results yielding the words in a human readable
    output.
    """
    fail = False
    misspelt = set()
    for results in all_results:
        if results.error:
            fail = True
            yield "ERROR: %s -- %s" % (results.context, results.error)
        elif results.words:
            fail = True
            misspelt.update(results.words)
            if display_context:
                yield "Misspelled words:\n<%s> %s" % (results.category, results.context)
                yield "-" * 80
                for word in results.words:
                    yield word
                yield "-" * 80
                yield ""

    if fail:
        yield "!!!Spelling check failed!!!"
        if display_summary:
            yield "\n".join(sorted(misspelt))
        if display_help:
            yield """
If the spelling checker reports a spelling mistake which is actually a
deliberate choice an exemption can be made in a few ways:

* Words containing uppercase characters are assumed to be proper nouns and
  ignored.
* Escaping can be achieved through the use of back ticks ` around the word.
* Adding to a custom wordlist wordlist.txt or spelling_wordlist.txt found in any
  sub-directory.
* Adding to the global wordlist https://github.com/resplendent-dev/unanimous
"""
            if custom_wordlists:
                custom_wordlists_lines = "\n".join(
                    [f"* {wordlist}" for wordlist in custom_wordlists]
                )
                yield f"""
Note: The following existing custom wordlists were found in this project:
{custom_wordlists_lines}
"""


def run_spell_check(config, workingpath):
    """
    Perform the spell check and keep a record of spelling mistakes.
    """
    with get_config_context_manager(workingpath, config) as ctxt:
        all_results = list(
            pyspelling.spellcheck(
                str(ctxt.config),
                names=[],
                groups=[],
                binary="",
                sources=[],
                verbose=0,
                debug=False,
            )
        )
        return all_results, ctxt.custom_wordlists


# vim: set ft=python:



"""
Make sure aspell is installed.
"""

import os
import subprocess  # noqa=S404 # nosec
import sys

import distro
from whichcraft import which

HANDLERS = {
    "debian": [
        ["sudo", "--non-interactive", "apt-get", "update"],
        [
            "sudo",
            "--non-interactive",
            "apt-get",
            "install",
            "-y",
            "aspell",
            "aspell-en",
        ],
    ],
    "arch": [["sudo", "--non-interactive", "pacman", "-S", "aspell", "aspell-en"]],
    "rhel": [
        ["sudo", "--non-interactive", "yum", "install", "epel-release"],
        ["sudo", "--non-interactive", "yum", "install", "aspell", "aspell-en"],
    ],
    "fedora": [["sudo", "--non-interactive", "dnf", "install", "aspell", "aspell-en"]],
}

CI_ENVS = {
    "CI",
    "APPVEYOR",
    "TF_BUILD",
    "CIRCLECI",
    "CI_SERVER",
    "GITLAB_CI",
    "CONTINUOUS_INTEGRATION",
    "TRAVIS",
    "IS_CI",
    "IN_CI",
}


def ensure_dependencies():
    """
    Check for presence of aspell
    """
    if which("aspell") is not None:
        return
    if not is_in_ci():
        print(
            "aspell and aspell-en need to be installed. "
            " Environment does not appear to be inside a"
            " CI system so timidly refusing to continue.",
            file=sys.stderr,
        )
        sys.exit(1)
    distro_id = distro.id()
    distro_like = distro.like()
    try:
        try:
            handler = HANDLERS[distro_id]
        except KeyError:
            handler = HANDLERS[distro_like]
    except KeyError:
        print(
            f"Unsupported Distribution: {distro_id} please add me to "
            f"https://github.com/resplendent-dev/spelling/blob/master"
            f"/app/spelling/dependencies.py",
            file=sys.stderr,
        )
        sys.exit(1)
    else:
        for cmdargs in handler:
            subprocess.call(cmdargs)  # noqa=S603 # nosec


def is_in_ci():
    """
    Check common CI environment variables
    """
    return bool(CI_ENVS.intersection(os.environ.keys()))


if __name__ == "__main__":
    ensure_dependencies()



"""failure with processing filter."""
from __future__ import print_function

from pyspelling import filters


class FailureFilter(filters.Filter):
    """Raise error during processing."""

    @staticmethod
    def get_default_config():
        """Get default configuration."""
        return {}

    def filter(self, source_file, encoding):  # noqa A001
        """Fail processing."""
        raise Exception("Failure during filtering")

    def sfilter(self, source):
        """Fail processing."""
        raise Exception("Failure during filtering")


def get_plugin():
    """Return the filter."""

    return FailureFilter



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
`setuptools` Distribution for spelling
"""

# System  Imports
import codecs
import os
import re

# External Imports
from setuptools import find_packages, setup

PACKAGE_NAME = "spelling"
URL = "https://github.com/resplendent-dev/spelling"
GITHUB_ORG = "resplendent-dev"
GITHUB_REPO = "spelling"
RE_SUB = "(https://github.com/%s/%s/blob/master/\\g<1>)" % (GITHUB_ORG, GITHUB_REPO)


def load_include(fname, transform=False):
    """
    Read the contents of relative `README` file.
    """
    file_path = os.path.join(os.path.dirname(__file__), fname)
    with codecs.open(file_path, encoding="utf-8") as fobj:
        data = fobj.read()
        if not transform:
            return data
        markdown_fixed = re.sub("[(]([^)]*[.](?:md|rst))[)]", RE_SUB, data)
        rst_fixed = re.sub(
            "^[.][.] [_][`][^`]*[`][:] ([^)]*[.](?:md|rst))", RE_SUB, markdown_fixed
        )
        return rst_fixed


def read_version():
    """
    Read the contents of relative file.
    """
    file_path = os.path.join(os.path.dirname(__file__), PACKAGE_NAME, "version.py")
    regex = re.compile("__version__ = ['\"]([^'\"]*)['\"]")
    with codecs.open(file_path, encoding="utf-8") as fobj:
        for line in fobj:
            mobj = regex.match(line)
            if mobj:
                return mobj.group(1)
    raise Exception("Failed to read version")


setup(
    name=PACKAGE_NAME,
    version=read_version(),
    author="Tim Gates",
    author_email="tim.gates@iress.com",
    maintainer="Tim Gates",
    maintainer_email="tim.gates@iress.com",
    packages=find_packages(exclude=["tests"]),
    license="GPLv3+",
    description=load_include("short_description.txt")
    .replace("\n", " ")
    .replace("\r", "")
    .strip(),
    long_description=load_include("README.md", transform=True),
    long_description_content_type="text/markdown",
    python_requires=">=3.6",
    test_suite="tests",
    test_requires=[
        elem.strip()
        for elem in load_include("test_requires.txt").splitlines()
        if elem.strip()
    ],
    install_requires=[
        elem.strip()
        for elem in load_include("requirements.txt").splitlines()
        if elem.strip()
    ],
    url=URL,
    classifiers=[
        elem
        for elem in [
            "Development Status :: 4 - Beta",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: Implementation :: CPython",
            "Programming Language :: Python :: Implementation :: PyPy",
            "Operating System :: OS Independent",
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        ]
        if elem
    ],
    package_data={"spelling": [".pyspelling.yml"]},
    entry_points={"console_scripts": ["spelling=spelling.__main__:main",]},
)



"""
Tools to load the default `.pyspelling` config and update it with the
nonwords dictionary and the custom exclusions.
"""

import io
import pathlib
import shutil
import tempfile
from contextlib import contextmanager

import pkg_resources
import yaml
from unanimous.store import get_current_non_words
from wcmatch import glob


class ConfigContext:
    """
    Class to load the default `.pyspelling` config and update it with the
    nonwords dictionary and the custom exclusions.
    """

    def __init__(self, tmppath, config, workingpath):
        self.tmppath = tmppath
        self.origconfig = config
        self.workingpath = workingpath
        self.config = tmppath / ".pyspelling"
        self.wordlist = tmppath / "wordlist.txt"
        self.custom_wordlists = []
        self.init()

    @staticmethod
    def load(config=None):
        """
        Open the existing config
        """
        if config is None:
            config = pkg_resources.resource_filename(__name__, ".pyspelling.yml")
        with io.open(config, "r", encoding="utf-8") as fobj:
            return yaml.safe_load(fobj)

    @staticmethod
    def save(target, yamldata):
        """
        Save the updated config
        """
        yaml.safe_dump(target, yamldata)

    def init(self):
        """
        Prepare the updated config
        """
        data = self.load(self.origconfig)
        nonwords = get_current_non_words()
        with io.open(str(self.wordlist), "w", encoding="utf-8") as fobj:
            for nonword in nonwords:
                print(nonword, file=fobj)
        self.update(data)
        with io.open(str(self.config), "w", encoding="utf-8") as fobj:
            self.save(data, fobj)

    def update(self, data):
        """
        Reconfigure the loaded yaml data as required
        """
        for entry in data["matrix"]:
            wordlists = entry["dictionary"].get("wordlists", [])
            self.custom_wordlists.extend(self.get_custom_wordlists(wordlists))
            break
        for entry in data["matrix"]:
            updated = list(self.custom_wordlists)
            updated.append(str(self.wordlist))
            entry["dictionary"]["wordlists"] = updated
            entry["sources"] = [
                [
                    source.replace("${DIR}", str(self.workingpath))
                    for source in entry["sources"][0]
                ]
            ]

    def get_custom_wordlists(self, wordlists):
        """
        Scan for existing custom wordlists
        """
        updated = []
        for wordlist in wordlists:
            wordlist = wordlist.replace("${DIR}", str(self.workingpath))
            wordlist_iglob = glob.iglob(
                wordlist, flags=glob.N | glob.B | glob.G | glob.S | glob.O
            )
            matches = list(wordlist_iglob)
            updated.extend(matches)
        return updated


@contextmanager
def get_config_context_manager(workingpath, config=None):
    """
    Loads the default `.pyspelling` config or the one provided and then
    updates it with the nonwords dictionary and the custom exclusions in a
    context manager that cleans up on completion.
    """
    tmpdir = tempfile.mkdtemp()
    yield ConfigContext(pathlib.Path(tmpdir), config, workingpath)
    shutil.rmtree(tmpdir)



#!/usr/bin/env python
"""
Module load handler for execution via:

python -m spelling
"""
from __future__ import absolute_import, division, print_function

import io
import logging
import os
import pathlib
import sys

import click

from spelling.check import check_iter
from spelling.dependencies import ensure_dependencies
from spelling.version import __version__

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS, invoke_without_command=True)
@click.version_option(version=__version__)
@click.pass_context
@click.option("--display-context/--no-display-context", default=True)
@click.option("--display-summary/--no-display-summary", default=True)
@click.option("--display-help/--no-display-help", default=True)
@click.option("--config", default=None)
@click.option("--working-path", default=None)
@click.option("--json-path", default=None)
def main(  # pylint: disable=too-many-arguments,bad-continuation
    ctxt,
    display_context,
    display_summary,
    display_help,
    config,
    working_path,
    json_path,
):
    """
    Used to conveniently invoke spell checking with sensible defaults and
    command line arguments to change the behavior.
    """
    if ctxt.invoked_subcommand is None:
        run_invocation(
            display_context,
            display_summary,
            display_help,
            config,
            working_path,
            json_path,
        )


@main.command()
@click.option("--display-context/--no-display-context", default=True)
@click.option("--display-summary/--no-display-summary", default=True)
@click.option("--display-help/--no-display-help", default=True)
@click.option("--config", default=None)
@click.option("--working-path", default=None)
@click.option("--json-path", default=None)
def invoke(  # pylint: disable=bad-continuation,too-many-arguments
    display_context, display_summary, display_help, config, working_path, json_path
):
    """
    Invoke the spell checker
    """
    run_invocation(
        display_context, display_summary, display_help, config, working_path, json_path
    )


def run_invocation(  # pylint: disable=bad-continuation,too-many-arguments
    display_context, display_summary, display_help, config, working_path, json_path
):
    """
    Call spell checker
    """
    logging.basicConfig()
    try:
        ensure_dependencies()
        success = True
        if working_path is None:
            working_path = pathlib.Path(os.getcwd()).resolve()
        jsonfobj = None
        if json_path:
            jsonfobj = io.open(json_path, "w", encoding="utf-8")
        try:
            msg_iter = check_iter(
                display_context=display_context,
                display_summary=display_summary,
                display_help=display_help,
                config=config,
                workingpath=working_path,
                jsonfobj=jsonfobj,
            )
            for msg in msg_iter:
                print(msg)
                success = False
        finally:
            if jsonfobj is not None:
                jsonfobj.close()
        if not success:
            sys.exit(1)
        print("Spelling check passed :)")
    except Exception:
        logging.exception("Error during processing.")
        sys.exit(2)
        raise


if __name__ == "__main__":
    main()  # pylint: disable=no-value-for-parameter
# vim: set ft=python:
